// ModelSwarm.m
//--------------------------------------------------------------------
#import <activity.h>
#import <collections.h>
#import <random.h>
#import "ModelSwarm.h"
//--------------------------------------------------------------------

@implementation ModelSwarm

// These methods provide access to the objects inside the ModelSwarm.
// These objects are the ones visible to other classes via message call.
// In theory we could just let other objects use Probes to read our state,
// but message access is frequently more convenient.

//--------------------------------------------------------------------

+ createBegin: aZone
{

   ModelSwarm *obj;
   id <ProbeMap> probeMap;

TRACE1(printf("\n%8X ModelSwarm - entry createBegin\n",(int)self);)

   // in createBegin, we set up the simulation parameters
   // First, call our superclass createBegin - the return value is the
   // allocated Swarm object.

   obj = [super createBegin: aZone];

   // Now fill in various simulation parameters with default values.

   obj->numberOfAgents      = 2;
   obj->numberOfCustomers   = 1;
   obj->maxPrice            = 15;
   obj->substitutionRate    = 1;
     //   [uniformDblRand getDoubleWithMin: 0.1 withMax: 1];
   obj->directCost          = 3;
     //   [uniformIntRand getIntegerWithMin: 1 withMax: 5];
   obj->fixedCost           = 0;
     //   [uniformIntRand getIntegerWithMin: 0 withMax: 20];

   // Build a customized probe map. Without a probe map, the default
   // is to show all variables and messages. Here we choose to
   // customize the appearance of the probe, give a nicer interface.

   probeMap = [EmptyProbeMap createBegin: aZone];
   [probeMap setProbedClass: [self class]];
   probeMap = [probeMap createEnd];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "numberOfAgents" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "numberOfCustomers" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "maxPrice" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "substitutionRate" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "directCost" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "fixedCost" inClass: [self class]]];

   // Now install our custom probeMap into the probeLibrary.

   [probeLibrary setProbeMap: probeMap For: [self class]];

TRACE1(printf("%8X ModelSwarm - exit createBegin %8X\n\n",
	      (int)self,(int)obj);)

   return obj;

}
//--------------------------------------------------------------------
- createEnd
{

   id obj;

TRACE1(printf("\n%8X ModelSwarm - entry createEnd\n",(int)self);)

   eyeCatcher[0] = 'M';
   eyeCatcher[0] = 'O';
   eyeCatcher[0] = 'D';
   eyeCatcher[0] = 'S';

   obj = [super createEnd];

TRACE1(printf("%8X ModelSwarm - exit createEnd %8X\n\n",
	      (int)self,(int)obj);)

   return obj;

}

//--------------------------------------------------------------------

- buildObjects
{

   // Here we create instances for objects before createEnd, for each,
   // the initial topic values are set using a specific method

   Agent          * anAgent;
   Customer       * aCustomer;
   int i;

   CW_DataWarehouse * aDataWarehouse;
   CW_Interface     * anInterface;
   CW_RuleMaker     * aRuleMaker;
   CW_RuleMaster    * aRuleMaster;
   CW_Shuffler      * shuffler;
   PH_ParmWarehouse * aParmWarehouse;
   PH_Reader        * reader;
   EH_Dump          * dump;


TRACE1 (printf("\n%8X ModelSwarm - entry buildObjects \n",(int)self);)

   dump     = [EH_Dump  createBegin: self];
   dump     = [dump     createEnd];

   reader   = [PH_Reader createBegin: self];
   reader   = [reader    createEnd];

   shuffler = [CW_Shuffler createBegin: self];
   shuffler = [shuffler    createEnd];

   aParmWarehouse = [PH_ParmWarehouse createBegin: self];
   aParmWarehouse = [aParmWarehouse   createEnd];
                    [aParmWarehouse   loadFromFile: "Agent.prm"];

   aRuleMaker  = [CW_RuleMaker createBegin: self];
   aRuleMaker  = [aRuleMaker   createEnd];
                 [aRuleMaker   setDump:     dump];
                 [aRuleMaker   setShuffler: shuffler];

   aRuleMaster = [CW_RuleMaster createBegin:  self];
                 [aRuleMaster   setRuleMaker: aRuleMaker];
   aRuleMaster = [aRuleMaster   createEnd];
                 [aRuleMaster   setDump:      dump];
                 [aRuleMaster   setShuffler:  shuffler];

   shop       = [Shop createBegin: self];
                [shop setNumberOfAgent: numberOfAgents];
                [shop setMemory: 100];           
   shop       = [shop createEnd];



   // here we create a bunch of agents and dataWarehouses; each agent
   // needs an own dataWarehouse

   agentList = [List create: self];

   for (i=0;i<numberOfAgents;i++)
   {

      [shop setPriceOf: i to: [uniformIntRand getIntegerWithMin: 0
                                              withMax: maxPrice]];

      aDataWarehouse=[CW_DataWarehouse createBegin: self];
                     [aDataWarehouse setParmWarehouse: aParmWarehouse];
                     [aDataWarehouse setReader:        reader];
      aDataWarehouse=[aDataWarehouse   createEnd];

      anInterface = [CW_Interface createBegin:    self];
                    [anInterface  setEventsToMap: 4];
      anInterface = [anInterface  createEnd];

      anAgent = [Agent createBegin: self];

      [anAgent setRuleMaster:    aRuleMaster];
      [anAgent setShop:          shop];
      [anAgent setDataWarehouse: aDataWarehouse];
      [anAgent setInterface:     anInterface];
      [anAgent setNumber:        i];
      [anAgent setDirectCost:    directCost];
      [anAgent setFixedCost:     fixedCost];

      anAgent = [anAgent createEnd];

      [agentList addLast: anAgent];

   }


   // here we create a bunch of customers

   customerList = [List create: self];

   for (i=0;i<numberOfCustomers;i++)
   {

      aCustomer = [Customer createBegin: self];

      [aCustomer setSubstitutionRate: substitutionRate];
      [aCustomer setMaxPrice:         maxPrice];
      [aCustomer setShop:             shop];

      aCustomer = [aCustomer createEnd];

      [customerList addLast: aCustomer];

   }

TRACE1 (printf("%8X ModelSwarm - exit buildObjects %8X\n\n",
               (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- buildActions
{

   // Create the list of simulation actions. We put these in an action
   // group, because we want these actions to be executed in a specific
   // order, but these steps should take no (simulated) time. The
   // M(foo) means "The message called <foo>". You can send a message
   // To a particular object, or ForEach object in a collection.

TRACE1(printf("\n%8X ModelSwarm - entry buildActions\n",(int)self);)

   modelActions = [ActionGroup create: self];
   [modelActions createActionForEach: agentList
                             message: M(fixNewPrice)];
   [modelActions createActionForEach: agentList
                             message: M(modifyPrice)];
   [modelActions createActionForEach: customerList
                             message: M(buy)];
   [modelActions createActionForEach: agentList
                             message: M(computeProfit)];
   [modelActions createActionTo:      shop
                             message: M(computeSectorialData)];
   [modelActions createActionForEach: agentList
                             message: M(evaluateProfit)];

   // Then we create a schedule that executes the modelActions. modelActions
   // is an ActionGroup, by itself it has no notion of time. In order to
   // have it executed in time, we create a Schedule that says to use
   // the modelActions ActionGroup at particular times.
   // This schedule has a repeat interval of 1, it will loop every time step.
   // The action is executed at time 0 relative to the beginning of the loop.
   // This is a simple schedule, with only one action that is just
   // repeated every time. See mousetraps for more complicated schedules.

   modelSchedule = [Schedule createBegin: self];
   [modelSchedule setRepeatInterval: 1];
   modelSchedule = [modelSchedule createEnd];
   [modelSchedule at: 0 createAction: modelActions];

TRACE1(printf("%8X ModelSwarm - exit buildActions %8X\n\n",
              (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- activateIn: swarmContext
{

   // Here, we activate the swarm in the context passed in
   // Then we activate our schedule in ourselves

TRACE1(printf("\n%8X ModelSwarm - entry activateIn\n",(int)self);)

   [super activateIn: swarmContext];
   [modelSchedule activateIn: self];

TRACE1(printf("%8X ModelSwarm - exit activateIn %8X\n\n",
              (int)self,(int)[self getSwarmActivity]);)

   return [self getSwarmActivity];

}

//--------------------------------------------------------------------

- getAgentList
{

TRACE1(printf("\n%8X ModelSwarm - entry getAgentList\n",(int)self);)
TRACE1(printf("%8X ModelSwarm - exit getAgentList %8X\n\n",
              (int)self,(int)agentList);)

   return agentList;

}

//--------------------------------------------------------------------

- (int) getMaxPrice
{

TRACE1(printf("\n%8X ModelSwarm - entry getMaxPrice\n",(int)self);)
TRACE1(printf("%8X ModelSwarm - exit getMaxPrice %d\n\n",
              (int)self,maxPrice);)

   return maxPrice;

}

//--------------------------------------------------------------------

- (float) getSubstitutionRate
{

TRACE1(printf("\n%8X ModelSwarm - entry getSubstitutionRate\n",(int)self);)
TRACE1(printf("%8X ModelSwarm - exit getSubstitutionRate %f\n\n",
              (int)self,substitutionRate);)

   return substitutionRate;

}

//--------------------------------------------------------------------

- (int) getDirectCost
{

TRACE1(printf("\n%8X ModelSwarm - entry getDirectCost\n",(int)self);)
TRACE1(printf("%8X ModelSwarm - exit getDirectCost %d\n\n",
              (int)self,directCost);)

   return directCost;

}

//--------------------------------------------------------------------

- (int) getFixedCost
{

TRACE1(printf("\n%8X ModelSwarm - entry getFixedCost\n",(int)self);)
TRACE1(printf("%8X ModelSwarm - exit getFixedCost %d\n\n",
              (int)self,fixedCost);)

   return fixedCost;

}


//--------------------------------------------------------------------

@end
