// ObserverSwarm.m
//--------------------------------------------------------------------

#import <activity.h>
#import <simtoolsgui.h>
#import "ObserverSwarm.h"

@implementation ObserverSwarm

//--------------------------------------------------------------------

+ createBegin: aZone
{

   ObserverSwarm *obj;

   id <ProbeMap> probeMap;

TRACE0(printf("\n%8X ObserverSwarm - entry createBegin\n",(int)self);)

   // createBegin: here we set up the default simulation parameters.
   // Superclass createBegin to allocate ourselves.

   obj = [super createBegin: aZone];

   // Fill in the relevant parameters (only one, in this case).

   obj->displayFrequency = 1000;
   obj->printFrequency   = 1000;

   // Also, build a customized probe map. Without a probe map, the default
   // is to show all variables and messages. Here we choose to
   // customize the appearance of the probe, give a nicer interface.

   probeMap = [EmptyProbeMap createBegin: aZone];
   [probeMap setProbedClass: [self class]];
   probeMap = [probeMap createEnd];

   // Add in a bunch of variables, one per simulation parameters

   [probeMap addProbe: [probeLibrary getProbeForVariable:
              "displayFrequency" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
              "printFrequency" inClass: [self class]]];
   // Now install our custom probeMap into the probeLibrary.

   [probeLibrary setProbeMap: probeMap For: [self class]];

TRACE0(printf("%8X ObserverSwarm - exit createBegin %8X\n\n",
	      (int)self,(int)obj);)

   return obj;

}

//--------------------------------------------------------------------
//Here we set initial values of counters and constant eyeCatcher
//that will be useful to find the area in a memory-dump

- createEnd
{

   id obj;

TRACE0(printf("\n%8X ObserverSwarm - entry createEnd\n",(int)self);)

   eyeCatcher[0] = 'O';
   eyeCatcher[1] = 'b';
   eyeCatcher[2] = 's';
   eyeCatcher[3] = 'S';

   steps = cnt = cycle = 0;
   flag  = 1;
   obj = [super createEnd];

TRACE0(printf("%8X ObserverSwarm - exit createEnd %8X\n\n",
	      (int)self,(int)obj);)

   return obj;

}

//--------------------------------------------------------------------

- buildObjects
{

   int i;

TRACE0(printf("\n%8X ObserverSwarm - entry buildObjects\n",
	      (int)self);)

   [super buildObjects];

   // First, we create the model that we're actually observing. The
   // model is a subswarm of the observer.
   // We create in the modelSwarm in "self" which creates a modelSwarm
   // in it's *own* Zone within the observerSwarm.

   modelSwarm = [ModelSwarm create: self];

   // Now create probe objects on the model and ourselves. This gives a
   // simple user interface to let the user change parameters.

   CREATE_PROBE_DISPLAY (modelSwarm);
   CREATE_PROBE_DISPLAY (self);

   // Instruct the control panel to wait for a button event.
   // We halt here until someone hits a control panel button.
   // Now that we're using Probes, the user can set the parameters
   // in the ModelSwarm probe window - we halt here to allow
   // the user to change parameters.

   [controlPanel setStateStopped];

   // When the user hits "go" on the control panel, we resume here
   // OK - the user said "go" so we're ready to start

   [modelSwarm buildObjects];

   // Now get down to building our own display objects.

   priceGraph = [EZGraph createBegin: self];
   [priceGraph setTitle: "prices"];
   [priceGraph setAxisLabelsX: "Cycles" Y: "Prices"];
   priceGraph = [priceGraph createEnd];

   profitGraph = [EZGraph createBegin: self];
   [profitGraph setTitle: "profits"];
   [profitGraph setAxisLabelsX: "Cycles" Y: "Profits"];
   profitGraph = [profitGraph createEnd];

   agentList = [modelSwarm getAgentList];
   numberOfAgents = [agentList getCount];

   for(i=0;i<numberOfAgents;i++)
   {

      [priceGraph  createSequence: "prices"
                     withFeedFrom: [agentList atOffset: i]
                      andSelector: M(getPrice)];

      [profitGraph createSequence: "profits"
                     withFeedFrom: [agentList atOffset: i]
                      andSelector: M(getProfit)];

   }

// [self compute];

TRACE0(printf("%8X ObserverSwarm - exit buildObjects %8X\n\n",
                (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- buildActions
{

   int i;

TRACE0(printf("\n%8X ObserverSwarm - entry buildActions\n",(int)self);)

   // Create the actions necessary for the simulation.

   [super buildActions];

   // First, let our model swarm build its own schedule.

   [modelSwarm buildActions];

   // Create an ActionGroup for display.

   displayActions   = [ActionGroup create: self];
   controlActions   = [ActionGroup create: self];

   // Schedule up the methods to draw the display of the world

   [displayActions createActionTo: priceGraph  message: M(step)];
   [displayActions createActionTo: profitGraph message: M(step)];

   [controlActions createActionTo: self message: M(verify)];
   [controlActions createActionTo: actionCache message: M(doTkEvents)];

   // And the display schedule. Note the repeat interval is set from our
   // own Swarm data structure. Display is frequently the slowest part of a
   // simulation, so redrawing less frequently can be a help.
   displaySchedule = [Schedule createBegin: self];
   [displaySchedule setRepeatInterval: displayFrequency];
   displaySchedule = [displaySchedule createEnd];
   [displaySchedule at: 0 createAction: displayActions];
   for(i=0;i<displayFrequency;i++)
   [displaySchedule at: i createAction: controlActions];

TRACE0(printf("%8X ObserverSwarm - exit buildActions %8X\n\n",
              (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- activateIn: swarmContext
{

// activateIn: - activate the schedules so they're ready to run.

TRACE0(printf("\n%8X ObserverSwarm - entry activateIn\n",(int)self);)

   [super activateIn: swarmContext];

   // Activate the model swarm in ourselves. The model swarm is a
   // subswarm of the observer swarm.

   [modelSwarm activateIn: self];

   // Now activate our schedule in ourselves. This arranges for the
   // execution of the schedule we built.

   [displaySchedule activateIn: self];

   // Activate returns the swarm activity - the thing that's ready to run.

TRACE0(printf("%8X ObserverSwarm - exit activateIn %8X\n\n",
              (int)self,(int)[self getSwarmActivity]);)

   return [self getSwarmActivity];

}

//--------------------------------------------------------------------

- compute
{

   int i,j,q;

TRACE0(printf("\n%8X ObserverSwarm - entry compute\n",(int)self);)

   maxPrice = [modelSwarm getMaxPrice];
   substitutionRate = [modelSwarm getSubstitutionRate];
   directCost = [modelSwarm getDirectCost];
   fixedCost = [modelSwarm getFixedCost];

   for(i=0;i<=maxPrice;i++)
      for(j=0;j<=maxPrice;j++)
      {

         q = maxPrice - i + substitutionRate * j;
         payoff[i*maxPrice+j] = q * (i - directCost) - fixedCost;

      }

   for(i=0;i<=maxPrice;i++)
      if((payoff[i*maxPrice+i] > maxPayoff) || (i==0))
      {

         maxPayoff = payoff[i*maxPrice+i];
         optimalPrice=i;

      }

TRACE0(printf("\n%8X ObserverSwarm - exit compute %8X\n",
	      (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- printPayoff
{

   int i,j;

TRACE0(printf("\n%8X ObserverSwarm - entry printPayoff\n",(int)self);)

   printf("\n");
   for(i=0;i<=maxPrice;i++) printf("%d!",i);
   printf("\n");
   for(i=0;i<=maxPrice;i++)
   {

      printf("\n%d!",i);
      for(j=0;j<=maxPrice;j++) printf("%d!",payoff[i*maxPrice+j]);
      printf("\n !");
      for(j=0;j<=maxPrice;j++) printf("%d!",payoff[j*maxPrice+i]);

   }

   printf("\n");

TRACE0(printf("\n%8X ObserverSwarm - exit printPayoff %8X\n",
	      (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- verify
{

   int i,j=0,p[16],pr[16];
   Agent * anAgent;

TRACE0(printf("\n%8X ObserverSwarm - entry verify\n",(int)self);)

   if (steps == 0)
   {

      [self compute];
      if (printFrequency > 0) [self printPayoff];

   }
   steps++;

   for(i=0;i<numberOfAgents;i++)
   {

      anAgent = [agentList atOffset: i];
      p[i]    = [anAgent getPrice];
      pr[i]   = [anAgent getProfit];
      if (pr[i] == maxPayoff) j++;

   }

   if (j == numberOfAgents)
   {

      cnt = cnt + flag;
      if (cycle == 0) cycle = steps;

   }
   else cycle = cnt = 0;

   if ((cnt > 2) || (steps % printFrequency == 0))
   {

//      [agentList forEach: M(print)];
      for(i=0;i<numberOfAgents;i++)
         printf("%d!%f!%d!%d!%d!%d!%d!%d!%d|%d!%d\n",steps,
                 substitutionRate,maxPrice,directCost,fixedCost,
                 optimalPrice,maxPayoff,i,p[i],pr[i],
                 maxPayoff-pr[i]);

      if (cnt > 2) flag = cnt = 0;

   }

TRACE0(printf("\n%8X ObserverSwarm - exit verify %8X\n",
	      (int)self,(int)self);)

   return self;

}


//--------------------------------------------------------------------

@end
